#include <math.h>
#include "vrst_snfp.h"

typedef union {
	float  f;
	double d; 
	long double ld;
	t__uint ui;
	t__uint lh[3];
	char s[10];
} snfp_floating_union;

static char fp_str[256];

#if defined(_BIG_ENDIAN) 
#define CFP_TO_EFP(cfp, efp) \
    (efp)->low  = (cfp)->lh[1]; \
    (efp)->high = (cfp)->lh[0]; 
#define CFP_TO_EXT_EFP(cfp, efp) \
    (efp)->low  = (cfp)->lh[1]; \
    (efp)->high = (cfp)->lh[0]; \
    (efp)->ext  = (cfp)->lh[2]; 
#define EFP_TO_CFP(efp, cfp) \
    (cfp)->lh[1] = (efp)->low; \
    (cfp)->lh[0] = (efp)->high; 
#define EXT_EFP_TO_CFP(efp, cfp) \
    (cfp)->lh[1] = (efp)->low; \
    (cfp)->lh[0] = (efp)->high;\
    (cfp)->lh[2] = (efp)->ext; 
#else 
#define CFP_TO_EFP(cfp, efp) \
    (efp)->low  = (cfp)->lh[0]; \
    (efp)->high = (cfp)->lh[1]; 
#define CFP_TO_EXT_EFP(cfp, efp) \
    (efp)->low  = (cfp)->lh[0]; \
    (efp)->high = (cfp)->lh[1]; \
    (efp)->ext  = (cfp)->lh[2]; 
#define EFP_TO_CFP(efp, cfp) \
    (cfp)->lh[0] = (efp)->low; \
    (cfp)->lh[1] = (efp)->high; 
#define EXT_EFP_TO_CFP(efp, cfp) \
    (cfp)->lh[0] = (efp)->low; \
    (cfp)->lh[1] = (efp)->high;\
    (cfp)->lh[2] = (efp)->ext; 
#endif

SN_TYPE(string) eexfp_to_str(SN_TYPE(snfp_floating_point_s) efp) {
	snfp_floating_union rep[1];
	EXT_EFP_TO_CFP(efp, rep);
	sprintf(fp_str, efp->str_fmt, rep->ld);
	chop_zero_tail(fp_str);
	return ts1(fp_str);
}
SN_TYPE(string) edfp_to_str(SN_TYPE(snfp_floating_point_s) efp) {
	snfp_floating_union rep[1];
	EFP_TO_CFP(efp, rep);
/*
	if( DEBUG_ON(snfp) ) {
		sprintf(fp_str, "[vrst_snfp C module] edfp_to_str(): value to convert = %.8lf", rep->d); 
		sn_dout(fp_str);
	}
*/
	sprintf(fp_str, efp->str_fmt, rep->d);
	chop_zero_tail(fp_str);
	return ts1(fp_str);
}
SN_TYPE(string) esfp_to_str(SN_TYPE(snfp_floating_point_s) efp) {
	snfp_floating_union rep[1];
	EFP_TO_CFP(efp, rep);
	sprintf(fp_str, efp->str_fmt, rep->f);
	chop_zero_tail(fp_str);
	return ts1(fp_str);
}
char* chop_zero_tail(char* str) {
	char* end = str + strlen(str) ;
	while(*(--end) == '0');
	*(end+1) = 0;
	return str;
}

void c_str_to_efp(SN_TYPE(snfp_floating_point_s) efp)
{
	snfp_floating_union rep[1];
	sscanf(efp->str_rep,"%lf", rep);
	CFP_TO_EFP(rep, efp);
}	
void c_str_to_ext_efp(SN_TYPE(snfp_floating_point_s) efp)
{
	snfp_floating_union rep[1];
	sscanf(efp->str_rep,"%Lf", rep);
	CFP_TO_EXT_EFP(rep, efp);
}	
void snfp_fn(SN_TYPE(snfp_floating_point_function_s) efp)
{
	snfp_floating_union op1[1];
	snfp_floating_union op2[1];
	snfp_floating_union res[1];
	EFP_TO_CFP(efp->op1, op1);
	EFP_TO_CFP(efp->op2, op2);

	switch( efp->its_type ) {
		case SN_ENUM(snfp_efp_function_t, FADD) : {
			res->d = op1->d + op2->d ;
			break;
		}
		case SN_ENUM(snfp_efp_function_t, FSUB) : {
			res->d = op1->d - op2->d;
			break;
		}
		case SN_ENUM(snfp_efp_function_t, FMUL) : {
			res->d = op1->d * op2->d;
			break;
		}
		case SN_ENUM(snfp_efp_function_t, FDIV) : {
			res->d = op1->d / op2->d ;
			break;
		}
		case SN_ENUM(snfp_efp_function_t, ACOS) : {
			res->d = acos(op1->d);
			break;
		}
		case SN_ENUM(snfp_efp_function_t, ASIN) : {
			res->d = asin(op1->d);
			break;
		}
		case SN_ENUM(snfp_efp_function_t, ATAN) : {
			res->d = atan(op1->d);
			break;
		}
		case SN_ENUM(snfp_efp_function_t, COS) : {
			res->d = cos(op1->d);
			break;
		}
		case SN_ENUM(snfp_efp_function_t, SIN) : {
			res->d = sin(op1->d);
			break;
		}
		case SN_ENUM(snfp_efp_function_t, TAN) : {
			res->d = tan(op1->d);
			break;
		}
		case SN_ENUM(snfp_efp_function_t, COSH) : {
			res->d = cosh(op1->d);
			break;
		}
		case SN_ENUM(snfp_efp_function_t, SINH) : {
			res->d = sinh(op1->d);
			break;
		}
		case SN_ENUM(snfp_efp_function_t, TANH) : {
			res->d = tanh(op1->d);
			break;
		}
		case SN_ENUM(snfp_efp_function_t, CMP) : {
			res->d = op1->d - op2->d;
			break;
		}
		default : {
			printf("\n[vrst_snfp]: Unsupported type yet!");
			break;
		}
	}
	sprintf(fp_str, "[vrst_snfp C module]: exec with op1=%.8lf, op2=%.8lf, res=%.8lf", op1->d, op2->d, res->d); 
	DBG(snfp, fp_str);
	CFP_TO_EFP(res, efp->res);
}
/*
------------------------
-----  CODE END   ------
------------------------
*/
