<'
'>
File: 	vrst_snfp.e
Description:
	give the user the ability to use floating point type in e
Notes:
Created By: meir Thu Jan 11 00:07:17 IST 2001
			exp			mantisa
single		 8				23
double		11				52
extended	15				63
<'
	// used to demonstrate float operations
type  snfp_sign_t 						: [POS, NEG](bits:1);
type  snfp_floating_point_t 			: [SINGLE, DOUBLE, EXTENDED];
type  snfp_floating_point_number_t		: [NORMAL, DENORMAL, ZERO, INFINITY, SNaN, QNaN];
type  snfp_fmt_t : [F_FMT, E_FMT];
type  snfp_efp_function_t : [FADD, FSUB, FMUL, FDIV, ACOS, ASIN, ATAN, COS, SIN, TAN, 
							 COSH, SINH, TANH, ACOSH, ASINH, ATANH, EXP, LOG, LOG10,
							 CMP];	//  0: op1 == op2 
							 		// >0: op1 > op2
							 		// <0: op1 < op2
	// 									   
define 		SFP_EXP_MAX 	0xFF;
define 		SFP_MAN_MAX 	0xFFFF_F7;
define 		DFP_EXP_MAX 	0xFF7;
define 		DFP_MAN_MAX 	0xFFFF_F;
define 		EXFP_EXP_MAX 	0xFFF7;
define 		EXFP_MAN_MAX 	0xFFFF_FFF7;

'>
o floating point e model
	snfp is a shortcut for the e structure representing a floating point, i.e.,
		a snfp consists of:
			sign	
			exp
			mantisa
		depending on the presicion these fields may be of different length.
	efp	is the snfp number as it is laidout in memory, i.e., it consists of:
		low		: uint
		high	: uint
		extended: uint
	  were,
	  %{low} may represent the layout of a singel precision number (c float, 32 bits wide)
	  %{high, low}	may represent a double precision number	(c double, 64 bits)
	  %{ext, high, low}		may represent an extended precision fp number (c long double, 80 bits wide)
<'
struct snfp_floating_point_s {
	kind		: snfp_floating_point_t;
	number_type : snfp_floating_point_number_t;
	str_rep : string;			// string representation of the fp
	fmt_to_use	: snfp_fmt_t;
		// ifc spec 
	set(sr : string) is empty;
	--get(): string is empty;	// return the string representation of the fp
	as_bin() : list of bit is empty;			// returns the physical bits, i.e., %{high,low}
	lt(op2 : snfp_floating_point_s) : bool is empty;
	eq(op2 : snfp_floating_point_s) : bool is empty;
		//
};

struct snfp_floating_point_function_s {
	op1		: snfp_floating_point_s;
	op2		: snfp_floating_point_s;
	res		: snfp_floating_point_s;
	its_type : snfp_efp_function_t;
	do_fp() is empty;
	lt() : bool is empty;	// true if op1 < op2
	lte() : bool is empty;	// true if op1 <= op2
	eq() : bool is empty;	// true if op1 == op2
};
'>

<'
define <fpnum'exp> "[-]<num>.<num>" as {
	"<1><2>.<3>";
};
define <fp_str_assign'action> "SNFP <fpstruct'exp> = [-]<int'num>.<frac'num>" as {
	<fpstruct'exp>.set( "<2><3>.<4>" );
};

type  snfp_random_fp_kind : [ZERO_TO_ONE, NON_TYPED, BIT_TOGGLE];
'>
o Struct: 			snfp_rnd_fp_s 
o Description: 
	provides means for generation of random floating point numbers
	based on strings, i.e., one can choose the number of digits before and 
	after the decimal point and their values.
o Members:
	kind	
		designate a type for classification and generation of different 
		number categories.
	sign
		0	= +
		1	= -
	int_len
		number of digits before the decimal point
	integer[int_len] : list of byte
		each byte represent one digit of the integral part of the number.
	frac_len
		number of digits after the decimal point
	fraction[frac_len] : list of byte
		each item in the list represents one digit after the decimal point
	str_rep
		the string representation of the fp number
o Main Methods:
	to_str_rep() : string
		returns the string representation of the fp number
o Notes:
	examples:
	1. to create random numbers  0 < r < 1 with up to 7 digits after the decimal point
		- keep kind == ZERO_TO_ONE 
		- keep frac_len <= 7
	2.  
o Created By: meir Thu Nov 22 23:09:00 IST 2001
<'
struct snfp_rnd_fp_s  {
	kind	: snfp_random_fp_kind;
	!no_zeros: bool;
	!no_ones : bool;
	sign	: bit;
	int_len : uint;
		keep int_len <= 6;
		keep soft int_len == select {
			50 : 1;
			20 : 2;
			15 : 3;
			10 : 4;
			 4 : 5;
			 1 : 6;
		};
	integer[int_len] : list of byte;
		keep for each in integer {
			it in [48..57];
			(int_len > 1) and (index == 0) => it != 48; //first digit in the integral part != 0
		};
	frac_len	: uint;
		keep frac_len <= 15;
		keep soft frac_len == select {
			50 : [6..15];
			20 : 5;
			15 : 4;
			10 : 3;
			 5 : others;
		};
	fraction[frac_len]	: list of byte;
		keep for each in fraction {
			it in [48..57];
		};
	str_rep : string;
	to_str_rep() : string is empty;
	post_generate() is also {
		if integer is empty { integer.add(48); };	//put 0 before the decimal point 
		if fraction is empty { fraction.add(48); };
		str_rep = to_str_rep();
	};
	when ZERO_TO_ONE snfp_rnd_fp_s {
		keep int_len.reset_soft() ;
		keep soft int_len == select {
			999 : 0;
			  1 : 1;
		};
		keep int_len == 1 => (integer[0] == 49) and (frac_len == 0); 
		keep sign == 0;
		keep no_ones  => int_len == 0;
		keep no_zeros => frac_len >= 1 ;
	};
	when BIT_TOGGLE snfp_rnd_fp_s {
		keep for each in integer {
			prev == 0x1 => it == 0x4;
			prev == 0x2 => it == 0x8;
			prev == 0x4 => it == 0x1;
			prev == 0x8 => it == 0x2;
		};
		keep for each in fraction {
			prev == 0x1 => it == 0x4;
			prev == 0x2 => it == 0x8;
			prev == 0x4 => it == 0x1;
			prev == 0x8 => it == 0x2;
		};
	};
	keep soft kind == ZERO_TO_ONE;
};
											//and returns it
'>
------------------------
-----  CODE END   ------
------------------------

